package com.miaxis.app.controller.question;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.miaxis.common.constant.Constants;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Arrays;
import java.util.stream.Collectors;

import com.miaxis.common.exception.CustomException;
import com.miaxis.common.utils.SecurityUtils;
import com.miaxis.question.domain.QuestionCollection;
import com.miaxis.question.domain.QuestionInfo;
import com.miaxis.question.dto.*;
import com.miaxis.question.vo.QuestionWrongIdDateVo;
import com.miaxis.score.domain.ScoreInfo;
import io.swagger.annotations.*;
import com.miaxis.common.core.domain.Response;
import org.springframework.beans.BeanUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.ModelAttribute;
import com.miaxis.common.annotation.Log;
import com.miaxis.common.core.controller.BaseController;
import com.miaxis.common.enums.BusinessTypeEnum;
import com.miaxis.question.domain.QuestionWrong;
import com.miaxis.question.service.IQuestionWrongService;
import com.miaxis.common.utils.poi.ExcelUtil;
import com.miaxis.common.core.page.ResponsePageInfo;

/**
 * 【wrong】Controller
 *
 * @author miaxis
 * @date 2021-08-19
 */
@RestController
@RequestMapping(Constants.STUDENT_PREFIX+"/question/wrong")
@Api(tags={"【APP-错题】"})
public class QuestionWrongController extends BaseController{
    @Autowired
    private IQuestionWrongService questionWrongService;


    /**
     * 新增wrong
     */
    @PostMapping
    @ApiOperation("新增wrong")
    public Response<Integer> add(@RequestBody QuestionWrongDTO questionWrongDTO) {
        //查询该用户已收藏的题目列表
        QuestionWrongListDTO dto = new QuestionWrongListDTO();
        dto.setUserId(SecurityUtils.getLoginUser().getStudent().getId());
        List<QuestionWrong> list = questionWrongService.selectQuestionWrongList(dto);
        long count = list.stream().filter(o -> questionWrongDTO.getQuestionId().equals(o.getQuestionId()) && questionWrongDTO.getKm().equals(o.getKm())).count();
        if (count > 0) {
            //throw new CustomException("该错已加入错题集");
            Response response = new Response(200, "该错已加入错题集");
            return response;
        }
        QuestionWrong questionWrong = new QuestionWrong();
        BeanUtils.copyProperties(questionWrongDTO, questionWrong);
        questionWrong.setUserId(SecurityUtils.getLoginUser().getStudent().getId());
        return toResponse(questionWrongService.save(questionWrong) ? 1 : 0);
    }

    @PostMapping("wrongs")
    @ApiOperation("合并本机和云端错题")
    public Response<Integer> wrongs(@RequestBody QuestionWrongYunDTO wrong) {
        //查询该用户已收藏的题目列表
        QuestionWrongListDTO dto = new QuestionWrongListDTO();
        dto.setUserId(SecurityUtils.getLoginUser().getStudent().getId());
        List<QuestionWrong> dblist = questionWrongService.selectQuestionWrongList(dto);
        List<String> questionIdsStr = dblist.stream().map(o -> o.getQuestionId() + "," + o.getKm()).collect(Collectors.toList());

        List<QuestionWrong> qlist = new ArrayList<QuestionWrong>();
        for (QuestionWgYunDTO wgYunDTO : wrong.getWrongs()) {
            if (questionIdsStr.contains(wgYunDTO.getId() + "," + wrong.getKm())) {
                continue;
            }
            QuestionWrong questionWrong = new QuestionWrong();
            questionWrong.setUserId(SecurityUtils.getLoginUser().getStudent().getId());
            questionWrong.setKm(wrong.getKm());
            questionWrong.setQuestionId(wgYunDTO.getId());
            Timestamp crDate = new Timestamp(wgYunDTO.getTimestamp());
            questionWrong.setCreateTime(crDate);
            qlist.add(questionWrong);
        }
        if (qlist.isEmpty()) {
            Response response = new Response(200, "选中的错题都已添加！");
            return response;
        }
        return toResponse(questionWrongService.saveBatch(qlist) ? 1 : 0);
    }


    @PostMapping("bakWrongs")
    @ApiOperation("备份本机错题到云端")
    public Response bakWrongs(@RequestBody QuestionWrongYunDTO wrong) {
        //首先删除云端数据
        Long userId = SecurityUtils.getLoginUser().getStudent().getId();
        QueryWrapper<QuestionWrong> queryWrapper = new QueryWrapper<QuestionWrong>();
        queryWrapper.eq("user_id", userId);
        queryWrapper.eq("km", wrong.getKm());
        questionWrongService.remove(queryWrapper);

        //保存上传的错题
        List<QuestionWrong> qlist = new ArrayList<QuestionWrong>();
        for (QuestionWgYunDTO wgYunDTO : wrong.getWrongs()) {
            QuestionWrong questionWrong = new QuestionWrong();
            questionWrong.setUserId(userId);
            questionWrong.setKm(wrong.getKm());
            questionWrong.setQuestionId(wgYunDTO.getId());
            Timestamp crDate = new Timestamp(wgYunDTO.getTimestamp());
            questionWrong.setCreateTime(crDate);
            qlist.add(questionWrong);
        }
        if (qlist.isEmpty()) {
            Response response = new Response(200, "该错已加入错题集(无须提示给用户)");
            return response;
        }
        return toResponse(questionWrongService.saveBatch(qlist) ? 1 : 0);
    }


    @DeleteMapping("/cancel/{questionId}")
    @ApiOperation("删除错题")
    public Response<Integer> remove(
            @ApiParam(name = "questionId", value = "问题id", required = true)
            @PathVariable Long questionId
    ) {
        Long userId = SecurityUtils.getLoginUser().getStudent().getId();
        QueryWrapper<QuestionWrong> queryWrapper = new QueryWrapper<QuestionWrong>();
        queryWrapper.eq("user_id", userId);
        queryWrapper.eq("question_id", questionId);
        questionWrongService.remove(queryWrapper);
        return Response.success();

    }

    @DeleteMapping("/cancelAll")
    @ApiOperation("清空错题")
    public Response<Integer> removeAll(@RequestBody QuestionWrongDelDTO questionWrongDelDTO) {
        Long userId = SecurityUtils.getLoginUser().getStudent().getId();
        QueryWrapper<QuestionWrong> queryWrapper = new QueryWrapper<QuestionWrong>();
        queryWrapper.eq("user_id", userId);
        queryWrapper.eq("km", questionWrongDelDTO.getKm());
        questionWrongService.remove(queryWrapper);
        return Response.success();
    }


    @GetMapping("/wrongByUser")
    @ApiOperation("恢复云端错题到本机（根据用户获取错题）")
    public Response<List<QuestionWrongIdDateVo>> appWrongByUser(@ModelAttribute QuestionWrongListDTO dto) {
        dto.setUserId(SecurityUtils.getLoginUser().getStudent().getId());
        List<QuestionWrongIdDateVo> list = questionWrongService.selectWrongIdByUserId(dto);
        return Response.success(list);
    }


    @GetMapping("/wrongCountByUser")
    @ApiOperation("根据用户获取错题数")
    public Response<Integer> wrongCountByUser(@ModelAttribute QuestionWrongListDTO dto) {
        dto.setUserId(SecurityUtils.getLoginUser().getStudent().getId());
        Integer count = questionWrongService.selectWrongCountByUserId(dto);
        return Response.success(count);
    }


}
